class ColissimoWebservice {
    constructor(options) {
        options = options || {};
        const defaults = {
            shippingMethodsContainer: document.querySelector('#sylius-shipping-methods'),
            form: document.querySelector('#dedi_colissimo_search_pickup_points'),
            submitBtn: document.querySelector('#dedi_colissimo_search_pickup_points_btn_submit'),
            getData: () => {
                const data = {};
                data[this.options.form.id] = {};
                const fields = ['city', 'postalCode', 'country', '_token'];
                for (const i in fields) {
                    const fieldName = fields[i];
                    const fieldElement = this.options.form.querySelector(`#${this.options.form.id}_${fieldName}`);
                    if (fieldElement) {
                        data[this.options.form.id][fieldName] = fieldElement.value;
                    }
                }

                return data;
            },
            success: (json) => {
                document.querySelector('[data-colissimo-pickup-points]').innerHTML = json.html;
            },
            error: (json) => {
            },
        };
        this.options = Object.assign(defaults, options)

        if (null !== this.options.form) {
            this.options['url'] = this.options.form.dataset.url;
        }

        this.bindEvents();
        this.initializeSearchFormDisplay();
    }

    bindEvents() {
        const that = this;

        this.options.shippingMethodsContainer
            .querySelectorAll('[id^="sylius_checkout_select_shipping_shipments_"]')
            .forEach(function(element) {
               element.addEventListener('click', () => {
                   that.initializeSearchFormDisplay();
               })
            });

        if (this.options.form !== null && this.options.form.tagName === 'form') {
            this.options.form.addEventListener('submit', (event) => {
                event.preventDefault();
                this.fetchPickupPoints();
            });

            return;
        }

        if (this.options.submitBtn !== null) {
            this.options.submitBtn.addEventListener('click', (event) => {
                event.preventDefault();
                this.fetchPickupPoints();
            });
        }
    }

    initializeSearchFormDisplay() {
        this.options.shippingMethodsContainer
            .querySelectorAll('.search-pickup-points')
            .forEach(function(element) {
                const relatedRadioInput = element.previousElementSibling.querySelector('[id^="sylius_checkout_select_shipping_shipments_"]');
                if (relatedRadioInput.checked === false) {
                    element.style.display = 'none';
                    element.querySelector('[data-colissimo-pickup-points]').innerHTML = '';
                } else {
                    element.style.display = 'block';
                }
            })
    }

    fetchPickupPoints() {
        fetch(this.options.url, {
            method: 'POST',
            body: JSON.stringify(this.options.getData()),
            headers: {
                'Accept': 'application/json',
                'Content-Type': 'application/json'
            },
        })
            .then(response => {
                if (!response.ok) {
                    return response.text().then(text => {
                        throw Error(text)
                    });
                }
                return response;
            })
            .then(response => response.json())
            .then(json => {
                this.options.success(json);
            })
            .catch(error => {
                this.options.error(error);
            });
    }
}